@@ -45,6 +45,7 @@ billboard BB;
 
 namespace gl {
   
+bool outlinemode=false;
 bool glthread=false;
 bool initialize=true;
 
@@ -418,6 +419,10 @@ void home()
   initlighting();
 }
 
+void nodisplay()
+{
+}
+
 void quit() 
 {
 #ifdef HAVE_LIBOSMESA
@@ -435,10 +440,14 @@ void quit()
     home();
     Animate=getSetting<bool>("autoplay");
 #ifdef HAVE_PTHREAD
-    if(!interact::interactive || animating)
+    if(!interact::interactive || animating) {
+      idle();
+      glutDisplayFunc(nodisplay);
       endwait(readySignal,readyLock);
+    }
 #endif    
-    glutHideWindow();
+    if(interact::interactive)
+      glutHideWindow();
   } else {
     glutDestroyWindow(window);
     exit(0);
@@ -452,6 +461,7 @@ void mode()
     case 0:
       for(size_t i=0; i < Nlights; ++i) 
         glEnable(GL_LIGHT0+i);
+      outlinemode=false;
       glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
       gluNurbsProperty(nurb,GLU_DISPLAY_MODE,GLU_FILL);
       ++Mode;
@@ -459,11 +469,13 @@ void mode()
     case 1:
       for(size_t i=0; i < Nlights; ++i) 
         glDisable(GL_LIGHT0+i);
+      outlinemode=true;
       glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
       gluNurbsProperty(nurb,GLU_DISPLAY_MODE,GLU_OUTLINE_PATCH);
       ++Mode;
       break;
     case 2:
+      outlinemode=false;
       gluNurbsProperty(nurb,GLU_DISPLAY_MODE,GLU_OUTLINE_POLYGON);
       Mode=0;
       break;
@@ -502,18 +514,6 @@ void reshape0(int width, int height)
   glViewport(0,0,Width,Height);
 }
   
-void update() 
-{
-  lastzoom=Zoom;
-  glLoadIdentity();
-  double cz=0.5*(zmin+zmax);
-  glTranslatef(cx,cy,cz);
-  glMultMatrixf(Rotate);
-  glTranslatef(0,0,-cz);
-  setProjection();
-  glutPostRedisplay();
-}
-
 void windowposition(int& x, int& y, int width=Width, int height=Height)
 {
   pair z=getSetting<pair>("position");
@@ -596,32 +596,6 @@ void togglefitscreen()
   fitscreen();
 }
 
-void updateHandler(int)
-{
-  queueScreen=true;
-  update();
-  if(interact::interactive || !Animate) {
-    glutShowWindow();
-    glutShowWindow(); // Call twice to work around apparent freeglut bug.
-  }
-}
-
-void reshape(int width, int height)
-{
-  if(glthread) {
-    static bool initialize=true;
-    if(initialize) {
-      initialize=false;
-      Signal(SIGUSR1,updateHandler);
-    }
-  }
-  
-  if(capsize(width,height))
-    glutReshapeWindow(width,height);
- 
-  reshape0(width,height);
-}
-  
 void initTimer() 
 {
   gettimeofday(&lasttime,NULL);
@@ -634,18 +608,6 @@ void idleFunc(void (*f)())
   glutIdleFunc(f);
 }
 
-void animate() 
-{
-  Animate=!Animate;
-  if(Animate) {
-    if(Fitscreen == 2) {
-      togglefitscreen();
-      togglefitscreen();
-    }
-    update();
-  }
-}
-
 void screen()
 {
   if(glthread && !interact::interactive)
@@ -654,7 +616,6 @@ void screen()
 
 void nextframe(int) 
 {
-  glFinish();
 #ifdef HAVE_PTHREAD
   endwait(readySignal,readyLock);
 #endif    
@@ -701,6 +662,58 @@ void display()
   }
 }
 
+void update() 
+{
+  glutDisplayFunc(display);
+  Animate=getSetting<bool>("autoplay");
+  glutShowWindow();
+  lastzoom=Zoom;
+  glLoadIdentity();
+  double cz=0.5*(zmin+zmax);
+  glTranslatef(cx,cy,cz);
+  glMultMatrixf(Rotate);
+  glTranslatef(0,0,-cz);
+  setProjection();
+  glutPostRedisplay();
+}
+
+void updateHandler(int)
+{
+  queueScreen=true;
+  update();
+  if(interact::interactive || !Animate) {
+    glutShowWindow();
+  }
+}
+
+void animate() 
+{
+  Animate=!Animate;
+  if(Animate) {
+    if(Fitscreen == 2) {
+      togglefitscreen();
+      togglefitscreen();
+    }
+    update();
+  }
+}
+
+void reshape(int width, int height)
+{
+  if(glthread) {
+    static bool initialize=true;
+    if(initialize) {
+      initialize=false;
+      Signal(SIGUSR1,updateHandler);
+    }
+  }
+  
+  if(capsize(width,height))
+    glutReshapeWindow(width,height);
+ 
+  reshape0(width,height);
+}
+  
 void shift(int x, int y)
 {
   if(x > 0 && y > 0) {
@@ -737,10 +750,18 @@ void capzoom()
   
 }
 
+int menustatus=GLUT_MENU_NOT_IN_USE;
+
+void menuStatus(int status, int x, int y) 
+{
+  menustatus=status;
+}
+  
 void disableMenu() 
 {
   Menu=false;
-  glutDetachMenu(MenuButton);
+  if(menustatus == GLUT_MENU_NOT_IN_USE)
+    glutDetachMenu(MenuButton);
 }
 
 void zoom(int x, int y)
@@ -942,6 +963,19 @@ void mouse(int button, int state, int x, int y)
   int mod=glutGetModifiers();
   string Action=action(button,mod);
 
+  if(!Menu) {
+    if(mod == 0 && state == GLUT_UP && !Motion && Action == "zoom/menu") {
+      MenuButton=button;
+      glutMotionFunc(NULL);
+      glutTimerFunc(getSetting<Int>("doubleclick"),timeout,0);
+      glutAttachMenu(button);
+      Menu=true;
+      return;
+    } else Motion=false;
+  }
+  
+  disableMenu();
+  
   if(Action == "zoomin") {
     glutMotionFunc(NULL);
     mousewheel(0,1,x,y);
@@ -953,18 +987,6 @@ void mouse(int button, int state, int x, int y)
     return;
   }     
   
-  if(Menu) disableMenu();
-  else {
-    if(mod == 0 && state == GLUT_UP && !Motion && Action == "zoom/menu") {
-      MenuButton=button;
-      glutMotionFunc(NULL);
-      glutAttachMenu(button);
-      Menu=true;
-      glutTimerFunc(getSetting<Int>("doubleclick"),timeout,0);
-      return;
-    } else Motion=false;
-  }
-  
   if(state == GLUT_DOWN) {
     if(Action == "rotate" || Action == "rotateX" || Action == "rotateY") {
       arcball.mouse_down(x,Height-y);
@@ -1229,6 +1251,7 @@ void exportHandler(int=0)
   if(!Iconify && !offscreen)
     glutHideWindow();
 #endif  
+  glutDisplayFunc(nodisplay);
 }
 
 static bool glinitialize=true;
@@ -1284,7 +1307,7 @@ void init()
 #ifdef HAVE_LIBGLUT
   mem::vector<string> cmd;
   cmd.push_back(settings::argv0);
-  if(Iconify)
+  if(!interact::interactive && Iconify)
     cmd.push_back("-iconic");
   push_split(cmd,getSetting<string>("glOptions"));
   char **argv=args(cmd,true);
@@ -1397,10 +1420,10 @@ void glrender(const string& prefix, const picture *pic, const string& format,
   pair maxtile=getSetting<pair>("maxtile");
   maxTileWidth=(int) maxtile.getx();
   maxTileHeight=(int) maxtile.gety();
+  if(maxTileWidth <= 0) maxTileWidth=1024;
+  if(maxTileHeight <= 0) maxTileHeight=768;
 
   if(offscreen) {
-    if(maxTileWidth <= 0) maxTileWidth=1024;
-    if(maxTileHeight <= 0) maxTileHeight=768;
     screenWidth=maxTileWidth;
     screenHeight=maxTileHeight;
 
@@ -1551,8 +1574,10 @@ void glrender(const string& prefix, const picture *pic, const string& format,
 #endif      
       if(samples > 1) {
         if(settings::verbose > 1 && samples > 1)
-          cout << "Multisampling enabled with sample width " << samples << endl;
+          cout << "Multisampling enabled with sample width " << samples
+               << endl;
       }
+      glutDisplayFunc(display);
       glutShowWindow();
     } else if(!havewindow) {
       glutInitWindowSize(maxTileWidth,maxTileHeight);
@@ -1570,7 +1595,7 @@ void glrender(const string& prefix, const picture *pic, const string& format,
     
 #ifdef HAVE_LIBGLUT
   if(!offscreen) {
-    Animate=getSetting<bool>("autoplay");
+    Animate=getSetting<bool>("autoplay") && glthread;
   
     if(View) {
       if(!getSetting<bool>("fitscreen"))
@@ -1617,7 +1642,6 @@ void glrender(const string& prefix, const picture *pic, const string& format,
     initializedView=true;
 #endif    
     glutReshapeFunc(reshape);
-    glutDisplayFunc(display);
     glutKeyboardFunc(keyboard);
     glutMouseFunc(mouse);
   
@@ -1635,6 +1659,7 @@ void glrender(const string& prefix, const picture *pic, const string& format,
     glutAddMenuEntry("(r) Reverse",REVERSE);
     glutAddMenuEntry("( ) Step",STEP);
     glutAddMenuEntry("(q) Quit" ,QUIT);
+    glutMenuStatusFunc(menuStatus);
   
     for(size_t i=0; i < nbuttons; ++i) {
       int button=buttons[i];
